from collections import Counter

from sklearn.model_selection import train_test_split as split_data


# Name of file from which to read raw annotated data
INPUT_FILE = 'initial-annotation-combined.tsv'

# Annotator agreement ratio below which annotations will not be
# included in the data
AGREEMENT_THRESHOLD = 0.75

# Proportion of examples to place in training, development, and test
# subsets. Must sum up to one.
TRAIN_RATIO = 0.6
DEV_RATIO = 0.20
TEST_RATIO = 0.20


# Load in annotations from INPUT_FILE, filter out missing annotations
# (label '-', representing cases where a user did not annotate a text)
# and store other labels ('accept', reject', or 'ignore') in a
# dictionary indexed by the text.

annotations_by_text = {}
with open(INPUT_FILE) as f:
    header = next(f)
    for line in f:
        line = line.rstrip('\n')
        fields = line.split('\t')
        text_id, text, annotations = fields[0], fields[1], fields[2:]
        annotations_by_text[text] = annotations
        # Filter out missing annotations
        annotations = [a for a in annotations if a != '-']
        # Note: this assumes that the texts are unique
        annotations_by_text[text] = annotations


# Use Counter to count the number of times each label ('accept',
# 'reject', 'ignore') appears in the data, filter out cases where
# 1) the annotator agreement is below AGREEMENT_THRESHOLD or
# 2) the most common label is 'ignore'. Store the most common label
# for the rest.
# (See https://docs.python.org/3/library/collections.html#collections.Counter
# for information on the Counter class.)

filtered_label_by_text = {}
for text, annotations in annotations_by_text.items():
    total_count = len(annotations)
    annotations = Counter(annotations)
    most_common_label, most_common_count = annotations.most_common()[0]
    if (most_common_count / total_count >= AGREEMENT_THRESHOLD and
        most_common_label != 'ignore'):
        filtered_label_by_text[text] = most_common_label
    
print(f'Filtered {len(annotations_by_text)} to {len(filtered_label_by_text)} '
      f'by agreement threshold {AGREEMENT_THRESHOLD}')


# Split filtered data into training, development and test sets
# according to the specified ratios so that the ratio of 'accept' and
# 'reject' labels in the sets is roughly equal to the ratio in the
# full dataset ("stratified")

if TRAIN_RATIO + DEV_RATIO + TEST_RATIO != 1.0:
    raise ValueError(f'TRAIN_RATIO {TRAIN_RATIO}, DEV_RATIO {DEV_RATIO} and'
                     f'TEST_RATIO {TEST_RATIO} do not add up to 1')

all_texts = list(filtered_label_by_text.keys())
all_labels = list(filtered_label_by_text.values())
train_size = int(len(all_texts) * TRAIN_RATIO)
dev_size = int(len(all_texts) * DEV_RATIO)
test_size = len(all_texts) - train_size - dev_size

train_texts, dt_texts, train_labels, dt_labels = split_data(
    all_texts, all_labels, train_size=train_size, stratify=all_labels)

dev_texts, test_texts, dev_labels, test_labels = split_data(
    dt_texts, dt_labels, test_size=test_size, stratify=dt_labels)


# Save the subsets in train.tsv, dev.tsv and test.tsv, printing
# out their statistics.

for name, texts, labels in (('train', train_texts, train_labels),
                            ('dev', dev_texts, dev_labels),
                            ('test', test_texts, test_labels)):
    counts = Counter(labels)
    ratios = { label: count/len(labels) for label, count in counts.items() }
    print(f'Placed {len(labels)} examples in {name} set: {ratios}')
    with open(f'{name}.tsv', 'w') as out:
        for text, label in zip(texts, labels):
            print(f'{label}\t{text}', file=out)
